@@ -39,6 +39,7 @@
 #include "ide-debug.h"
 #include "ide-log.h"
 #include "ide-macros.h"
+#include "ide-private.h"
 
 /**
  * SECTION:ide-log
@@ -211,6 +212,10 @@ ide_log_handler (const gchar    *log_domain,
   if (g_strcmp0 ("GdkPixbuf", log_domain) == 0)
     return;
 
+  /* Let tracer know about log message */
+  if (log_level < IDE_LOG_LEVEL_TRACE)
+    _ide_trace_log (log_level, log_domain, message);
+
   if (G_LIKELY (channels->len))
     {
       is_debug_level = (log_level == G_LOG_LEVEL_DEBUG || log_level == IDE_LOG_LEVEL_TRACE);
