@@ -981,6 +981,17 @@ ide_workbench_load_project_completed (IdeWorkbench *self,
   g_assert (lp->addins != NULL);
   g_assert (lp->addins->len == 0);
 
+  /* If we did not get a VCS as part of the loading process, set the
+   * fallback VCS implementation.
+   */
+  if (self->vcs == NULL)
+    {
+      g_autoptr(GFile) workdir = ide_context_ref_workdir (self->context);
+      g_autoptr(IdeDirectoryVcs) vcs = ide_directory_vcs_new (workdir);
+
+      ide_workbench_set_vcs (self, IDE_VCS (vcs));
+    }
+
   if (lp->workspace_type != G_TYPE_INVALID)
     {
       IdeWorkspace *workspace;
@@ -1228,6 +1239,20 @@ ide_workbench_load_project_async (IdeWorkbench        *self,
   name = g_file_get_basename (directory);
   ide_context_set_title (self->context, name);
 
+  {
+    GFile *pdir = ide_project_info_get_directory (project_info);
+    GFile *pfile = ide_project_info_get_file (project_info);
+    const gchar *pident = ide_project_info_get_id (project_info);
+    const gchar *pname = ide_project_info_get_name (project_info);
+
+    /* Log some information to help track down project loading issues. */
+    g_debug ("Loading project");
+    g_debug ("    id = %s", pname);
+    g_debug ("  name = %s", pident);
+    g_debug ("   dir = %s", g_file_peek_path (pdir));
+    g_debug ("  file = %s", g_file_peek_path (pfile));
+  }
+
   /* If there has not been a project name set, make the default matching
    * the directory name. A plugin may update the name with more information
    * based on .doap files, etc.
@@ -1443,6 +1468,32 @@ ide_workbench_get_project_info (IdeWorkbench *self)
   return self->project_info;
 }
 
+static void
+ide_workbench_unload_foundry_cb (GObject      *object,
+                                 GAsyncResult *result,
+                                 gpointer      user_data)
+{
+  g_autoptr(IdeTask) task = user_data;
+  g_autoptr(GError) error = NULL;
+  IdeWorkbench *self;
+
+  g_assert (G_IS_ASYNC_RESULT (result));
+  g_assert (IDE_IS_TASK (task));
+
+  self = ide_task_get_source_object (task);
+
+  if (!_ide_foundry_unload_finish (result, &error))
+    ide_task_return_error (task, g_steal_pointer (&error));
+  else
+    ide_task_return_boolean (task, TRUE);
+
+  if (self->context != NULL)
+    {
+      ide_object_destroy (IDE_OBJECT (self->context));
+      g_clear_object (&self->context);
+    }
+}
+
 static void
 ide_workbench_unload_project_completed (IdeWorkbench *self,
                                         IdeTask      *task)
@@ -1453,13 +1504,10 @@ ide_workbench_unload_project_completed (IdeWorkbench *self,
   g_clear_object (&self->addins);
   ide_workbench_foreach_workspace (self, (GtkCallback)gtk_widget_destroy, NULL);
 
-  if (self->context != NULL)
-    {
-      ide_object_destroy (IDE_OBJECT (self->context));
-      g_clear_object (&self->context);
-    }
-
-  ide_task_return_boolean (task, TRUE);
+  _ide_foundry_unload_async (self->context,
+                             ide_task_get_cancellable (task),
+                             ide_workbench_unload_foundry_cb,
+                             g_object_ref (task));
 }
 
 static void
@@ -1617,7 +1665,6 @@ ide_workbench_unload_async (IdeWorkbench        *self,
                           cancellable,
                           ide_workbench_session_save_cb,
                           g_steal_pointer (&task));
-
 }
 
 /**
@@ -1861,7 +1908,12 @@ sort_by_priority (gconstpointer a,
   if (!ide_workbench_addin_can_open (addin_b, o->file, o->content_type, &prio_b))
     return -1;
 
-  return prio_a - prio_b;
+  if (prio_a < prio_b)
+    return -1;
+  else if (prio_a > prio_b)
+    return 1;
+  else
+    return 0;
 }
 
 static void
@@ -2216,7 +2268,7 @@ ide_workbench_set_vcs (IdeWorkbench *self,
   g_return_if_fail (IDE_IS_WORKBENCH (self));
   g_return_if_fail (!vcs || IDE_IS_VCS (vcs));
 
-  if (vcs == self->vcs)
+  if (self->vcs != NULL && vcs == self->vcs)
     return;
 
   if (vcs == NULL)
