@@ -84,8 +84,6 @@ struct _IdeFlatpakSubprocess
 
   GMainContext *main_context;
 
-  guint sigint_id;
-  guint sigterm_id;
   guint exited_subscription;
 
   /* GList of GTasks for wait_async() */
@@ -923,54 +921,6 @@ subprocess_iface_init (IdeSubprocessInterface *iface)
   iface->communicate_utf8_finish = ide_flatpak_subprocess_communicate_utf8_finish;
 }
 
-static gboolean
-sigterm_handler (gpointer user_data)
-{
-  IdeFlatpakSubprocess *self = user_data;
-
-  IDE_ENTRY;
-
-  g_assert (IDE_IS_FLATPAK_SUBPROCESS (self));
-
-  g_dbus_connection_call_sync (self->connection,
-                               "org.freedesktop.Flatpak",
-                               "/org/freedesktop/Flatpak/Development",
-                               "org.freedesktop.Flatpak.Development",
-                               "HostCommandSignal",
-                               g_variant_new ("(uub)", self->client_pid, SIGTERM, TRUE),
-                               NULL,
-                               G_DBUS_CALL_FLAGS_NONE, -1,
-                               NULL, NULL);
-
-  kill (getpid (), SIGTERM);
-
-  IDE_RETURN (G_SOURCE_CONTINUE);
-}
-
-static gboolean
-sigint_handler (gpointer user_data)
-{
-  IdeFlatpakSubprocess *self = user_data;
-
-  IDE_ENTRY;
-
-  g_assert (IDE_IS_FLATPAK_SUBPROCESS (self));
-
-  g_dbus_connection_call_sync (self->connection,
-                               "org.freedesktop.Flatpak",
-                               "/org/freedesktop/Flatpak/Development",
-                               "org.freedesktop.Flatpak.Development",
-                               "HostCommandSignal",
-                               g_variant_new ("(uub)", self->client_pid, SIGINT, TRUE),
-                               NULL,
-                               G_DBUS_CALL_FLAGS_NONE, -1,
-                               NULL, NULL);
-
-  kill (getpid (), SIGINT);
-
-  IDE_RETURN (G_SOURCE_CONTINUE);
-}
-
 static void
 maybe_create_input_stream (GInputStream **ret,
                            gint          *fdptr,
@@ -1046,10 +996,6 @@ ide_flatpak_subprocess_complete_command_locked (IdeFlatpakSubprocess *self,
   self->client_pid = 0;
   g_clear_pointer (&self->identifier, g_free);
 
-  /* Remove our sources used for signal propagation */
-  g_clear_handle_id (&self->sigint_id, g_source_remove);
-  g_clear_handle_id (&self->sigterm_id, g_source_remove);
-
   /* Complete async workers */
   waiting = g_steal_pointer (&self->waiting);
 
@@ -1111,10 +1057,10 @@ host_command_exited_cb (GDBusConnection *connection,
                  (guint)self->client_pid,
                  (guint)exit_status);
 
-  /* We can release our dbus signal handler now */
+  /* We can release our D-Bus signal handler now */
   if (self->exited_subscription != 0)
     {
-      IDE_TRACE_MSG ("Unsubscribing from DBus subscription %d", self->exited_subscription);
+      IDE_TRACE_MSG ("Unsubscribing from D-Bus subscription %d", self->exited_subscription);
       g_dbus_connection_signal_unsubscribe (self->connection, self->exited_subscription);
       self->exited_subscription = 0;
     }
@@ -1199,37 +1145,9 @@ ide_flatpak_subprocess_initable_init (GInitable     *initable,
   g_assert (IDE_IS_FLATPAK_SUBPROCESS (self));
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  /*
-   * FIXME:
-   *
-   * Because we are seeing a rather difficult to track down bug where we lose
-   * the connection upon submission of the HostCommand() after a timeout period
-   * (the dbus-daemon is closing our connection) we are using a private
-   * GDBusConnection for the command.
-   *
-   * This means we need to ensure we close the connection as soon as we can
-   * so that we don't hold things open for too long. Additionally, if we do
-   * get disconnected from the daemon, we don't want to crash but instead will
-   * synthesize the completion of the command. However, this should be much
-   * more unlikely since we haven't seen this failure case.
-   *
-   * One thing we could look into for recovery is to send a SIGKILL to a new
-   * connection (of our client pid) during recovery to ensure that it dies and
-   * force our operation to fail. Callers could handle this during wait_async()
-   * wait_finish() pairs. Again, not ideal.
-   */
-  self->connection =
-    g_dbus_connection_new_for_address_sync (g_getenv ("DBUS_SESSION_BUS_ADDRESS"),
-                                            G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT | G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
-                                            NULL,
-                                            cancellable,
-                                            error);
-
-  if (self->connection == NULL)
+  if (!(self->connection = g_bus_get_sync (G_BUS_TYPE_SESSION, cancellable, error)))
     IDE_RETURN (FALSE);
 
-  g_dbus_connection_set_exit_on_close (self->connection, FALSE);
-
   if (self->clear_env)
     flags |= FLATPAK_HOST_COMMAND_FLAGS_CLEAR_ENV;
 
@@ -1413,18 +1331,9 @@ ide_flatpak_subprocess_initable_init (GInitable     *initable,
     }
 
 
-  /*
-   * Register signal handlers for SIGTERM/SIGINT so that we can terminate
-   * the host process with us (which won't be guaranteed since its outside
-   * our cgroup, nor can we use a process group leader).
-   */
-  self->sigterm_id = g_unix_signal_add (SIGTERM, sigterm_handler, self);
-  self->sigint_id = g_unix_signal_add (SIGINT, sigint_handler, self);
-
-
   /*
    * Make sure we've closed or stolen all of the FDs that are in play
-   * before calling the DBus service.
+   * before calling the D-Bus service.
    */
   g_assert_cmpint (-1, ==, stdin_pair[0]);
   g_assert_cmpint (-1, ==, stdin_pair[1]);
@@ -1558,7 +1467,7 @@ ide_flatpak_subprocess_dispose (GObject *object)
     {
       if (self->connection != NULL && !g_dbus_connection_is_closed (self->connection))
         {
-          IDE_TRACE_MSG ("Unsubscribing from DBus subscription %d", self->exited_subscription);
+          IDE_TRACE_MSG ("Unsubscribing from D-Bus subscription %d", self->exited_subscription);
           g_dbus_connection_signal_unsubscribe (self->connection, self->exited_subscription);
         }
 
@@ -1568,9 +1477,6 @@ ide_flatpak_subprocess_dispose (GObject *object)
   if (self->waiting != NULL)
     g_warning ("improper disposal while async operations are active!");
 
-  g_clear_handle_id (&self->sigint_id, g_source_remove);
-  g_clear_handle_id (&self->sigterm_id, g_source_remove);
-
   G_OBJECT_CLASS (ide_flatpak_subprocess_parent_class)->dispose (object);
 }
 
@@ -1582,8 +1488,6 @@ ide_flatpak_subprocess_finalize (GObject *object)
   IDE_ENTRY;
 
   g_assert (self->waiting == NULL);
-  g_assert_cmpint (self->sigint_id, ==, 0);
-  g_assert_cmpint (self->sigterm_id, ==, 0);
   g_assert_cmpint (self->exited_subscription, ==, 0);
 
   g_clear_pointer (&self->identifier, g_free);
