@@ -31,13 +31,88 @@
 #include <libide-greeter.h>
 #include <libide-gui.h>
 #include <libide-threading.h>
+#ifdef ENABLE_TRACING_SYSCAP
+# include <sysprof-capture.h>
+#endif
+#include <sched.h>
+#include <unistd.h>
 
 #include "ide-application-private.h"
+#include "ide-debug.h"
 #include "ide-thread-private.h"
 #include "ide-terminal-private.h"
+#include "ide-private.h"
 
 #include "bug-buddy.h"
 
+#ifdef ENABLE_TRACING_SYSCAP
+static SysprofCaptureWriter *trace_writer;
+static G_LOCK_DEFINE (tracer);
+
+static void
+trace_load (void)
+{
+  sysprof_clock_init ();
+  trace_writer = sysprof_capture_writer_new_from_env (0);
+}
+
+static void
+trace_unload (void)
+{
+  if (trace_writer)
+    {
+      sysprof_capture_writer_flush (trace_writer);
+      g_clear_pointer (&trace_writer, sysprof_capture_writer_unref);
+    }
+}
+
+static void
+trace_function (const gchar    *func,
+                gint64          begin_time_usec,
+                gint64          end_time_usec)
+{
+  if (trace_writer != NULL)
+    {
+      G_LOCK (tracer);
+      sysprof_capture_writer_add_mark (trace_writer,
+                                       begin_time_usec * 1000L,
+                                       sched_getcpu (),
+                                       getpid (),
+                                       (end_time_usec - begin_time_usec) * 1000L,
+                                       "tracing",
+                                       "function",
+                                       func);
+      G_UNLOCK (tracer);
+    }
+}
+
+static void
+trace_log (GLogLevelFlags  log_level,
+           const gchar    *domain,
+           const gchar    *message)
+{
+  if (trace_writer != NULL)
+    {
+      G_LOCK (tracer);
+      sysprof_capture_writer_add_log (trace_writer,
+                                      SYSPROF_CAPTURE_CURRENT_TIME,
+                                      sched_getcpu (),
+                                      getpid (),
+                                      log_level,
+                                      domain,
+                                      message);
+      G_UNLOCK (tracer);
+    }
+}
+
+static IdeTraceVTable trace_vtable = {
+  trace_load,
+  trace_unload,
+  trace_function,
+  trace_log,
+};
+#endif
+
 static gboolean
 verbose_cb (const gchar  *option_name,
             const gchar  *value,
@@ -115,6 +190,12 @@ main (gint   argc,
   /* Always ignore SIGPIPE */
   signal (SIGPIPE, SIG_IGN);
 
+  /* Set up gettext translations */
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  textdomain (GETTEXT_PACKAGE);
+
   /* Setup various application name/id defaults. */
   g_set_prgname (ide_get_program_name ());
   g_set_application_name (_("Builder"));
@@ -159,12 +240,18 @@ main (gint   argc,
   if (desktop == NULL)
     desktop = "unknown";
 
+#ifdef ENABLE_TRACING_SYSCAP
+  _ide_trace_init (&trace_vtable);
+#endif
+
   g_message ("Initializing with %s desktop and GTK+ %d.%d.%d.",
              desktop,
              gtk_get_major_version (),
              gtk_get_minor_version (),
              gtk_get_micro_version ());
 
+  gtk_source_init ();
+
   /* Initialize thread pools */
   _ide_thread_pool_init (FALSE);
 
@@ -184,8 +271,11 @@ main (gint   argc,
   ide_log_shutdown ();
 
   /* Cleanup GtkSourceView singletons to improve valgrind output */
-  g_object_unref (gtk_source_style_scheme_manager_get_default ());
-  g_object_unref (gtk_source_language_manager_get_default ());
+  gtk_source_finalize ();
+
+#ifdef ENABLE_TRACING_SYSCAP
+  _ide_trace_shutdown ();
+#endif
 
   return ret;
 }
