@@ -244,7 +244,7 @@ ipc_git_repository_impl_handle_path_is_ignored (IpcGitRepository      *repositor
 }
 
 static gint
-compare_tags (gconstpointer a,
+compare_refs (gconstpointer a,
               gconstpointer b)
 {
   return g_utf8_collate (*(const gchar **)a, *(const gchar **)b);
@@ -293,14 +293,13 @@ ipc_git_repository_impl_handle_list_refs_by_kind (IpcGitRepository      *reposit
       if (!(names = ggit_repository_list_tags (self->repository, &error)))
         return complete_wrapped_error (invocation, error);
 
-      qsort (names, g_strv_length (names), sizeof (gchar *), compare_tags);
-
       for (guint i = 0; names[i] != NULL; i++)
         g_ptr_array_add (ret, g_steal_pointer (&names[i]));
     }
   else
     g_assert_not_reached ();
 
+  qsort (ret->pdata, ret->len, sizeof (gchar *), compare_refs);
   g_ptr_array_add (ret, NULL);
 
   ipc_git_repository_complete_list_refs_by_kind (repository,
@@ -751,26 +750,11 @@ push_worker (GTask        *task,
   g_assert (GGIT_IS_REMOTE (push->remote));
   g_assert (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-  ggit_remote_connect (push->remote,
-                       GGIT_DIRECTION_PUSH,
-                       push->callbacks,
-                       push->proxy_options,
-                       NULL,
-                       &error);
-
-  if (error != NULL)
-    {
-      g_task_return_error (task, g_steal_pointer (&error));
-      return;
-    }
-
   ggit_remote_push (push->remote,
                     (const gchar * const *)push->ref_names,
                     push->push_options,
                     &error);
 
-  ggit_remote_disconnect (push->remote);
-
   if (error != NULL)
     g_task_return_error (task, g_steal_pointer (&error));
   else
@@ -856,9 +840,10 @@ ipc_git_repository_impl_handle_push (IpcGitRepository      *repository,
   callbacks = ipc_git_remote_callbacks_new (progress);
 
   push_options = ggit_push_options_new ();
-  ggit_push_options_set_callbacks (push_options, callbacks);
+  ggit_push_options_set_remote_callbacks (push_options, callbacks);
 
   push = g_slice_new0 (Push);
+  push->ref_names = g_strdupv ((gchar **)ref_names);
   push->remote = g_steal_pointer (&remote);
   push->progress = g_steal_pointer (&progress);
   push->callbacks = g_steal_pointer (&callbacks);
